@@ -18,7 +18,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-use crate::transport::{Transport, TransportError, ListenerEvent};
+use crate::transport::{ListenerEvent, Transport, TransportError};
 use futures::prelude::*;
 use multiaddr::Multiaddr;
 use std::{error, pin::Pin, task::Context, task::Poll};
@@ -53,14 +53,28 @@ where
         let map = self.map;
         match self.transport.listen_on(addr) {
             Ok(stream) => Ok(MapErrListener { inner: stream, map }),
-            Err(err) => Err(err.map(map))
+            Err(err) => Err(err.map(map)),
         }
     }
 
     fn dial(self, addr: Multiaddr) -> Result<Self::Dial, TransportError<Self::Error>> {
         let map = self.map;
         match self.transport.dial(addr) {
-            Ok(future) => Ok(MapErrDial { inner: future, map: Some(map) }),
+            Ok(future) => Ok(MapErrDial {
+                inner: future,
+                map: Some(map),
+            }),
+            Err(err) => Err(err.map(map)),
+        }
+    }
+
+    fn dial_as_listener(self, addr: Multiaddr) -> Result<Self::Dial, TransportError<Self::Error>> {
+        let map = self.map;
+        match self.transport.dial_as_listener(addr) {
+            Ok(future) => Ok(MapErrDial {
+                inner: future,
+                map: Some(map),
+            }),
             Err(err) => Err(err.map(map)),
         }
     }
@@ -92,11 +106,9 @@ where
             Poll::Ready(Some(Ok(event))) => {
                 let map = &*this.map;
                 let event = event
-                    .map(move |value| {
-                        MapErrListenerUpgrade {
-                            inner: value,
-                            map: Some(map.clone())
-                        }
+                    .map(move |value| MapErrListenerUpgrade {
+                        inner: value,
+                        map: Some(map.clone()),
                     })
                     .map_err(|err| (map.clone())(err));
                 Poll::Ready(Some(Ok(event)))
@@ -117,7 +129,8 @@ pub struct MapErrListenerUpgrade<T: Transport, F> {
 }
 
 impl<T, F, TErr> Future for MapErrListenerUpgrade<T, F>
-where T: Transport,
+where
+    T: Transport,
     F: FnOnce(T::Error) -> TErr,
 {
     type Output = Result<T::Output, TErr>;
